@@ -28,6 +28,8 @@ Reading radar data
     read_kazr
     read_noxp_iphex_nc
     read_odim_h5
+    read_odim_grid_h5
+    read_odim_vp_h5
     read_pattern
     read_radx
     read_rainbow_wrl
@@ -36,11 +38,17 @@ Reading radar data
     read_gif
     read_bin
     read_iq
+    read_bin_mf
+    read_png
+    read_dat_mf
+    read_grib
     read_rainbow_psr
     read_rainbow_psr_spectra
     read_spectra
     read_cf1
     read_cf1_cartesian
+    read_cf1_cartesian_mf
+    read_hpl
 
 Writing radar data
 ==================
@@ -59,7 +67,7 @@ from .d3r_gcpex_nc import read_d3r_gcpex_nc
 from .noxp_iphex_nc import read_noxp_iphex_nc
 from .arm_vpt import read_kazr
 from .edge_netcdf import read_edge_netcdf
-from .odim_h5 import read_odim_h5
+from .odim_h5 import read_odim_h5, read_odim_grid_h5, read_odim_vp_h5
 from .odim_h5_writer import write_odim_h5
 from .gamic_hdf5 import read_gamic
 from .sinarame_h5 import read_sinarame_h5
@@ -70,7 +78,7 @@ from .rainbow_psr import read_psr_header, read_psr_cpi_header, read_psr_spectra
 from .rainbow_psr import convert_data
 from .spectra import read_spectra, write_spectra
 from .cf1 import read_cf1
-from .cf1_cartesian import read_cf1_cartesian
+from .cf1_cartesian import read_cf1_cartesian, read_cf1_cartesian_mf
 
 from .metranet_reader import read_metranet
 from .metranet_cartesian_reader import read_cartesian_metranet
@@ -84,5 +92,10 @@ from .metranet_python import read_file as read_file_py
 
 from .rad4alp_gif_reader import read_gif
 from .rad4alp_bin_reader import read_bin
+from .mf_bin_reader import read_bin_mf
+from .mf_png_reader import read_png
+from .mf_grib_reader import read_grib
+from .mf_dat_reader import read_dat_mf
+from .hpl_reader import read_hpl
 
 __all__ = [s for s in dir() if not s.startswith('_')]
