@@ -14,7 +14,6 @@ Retrieval of QVPs from a radar object
     compute_svp
     compute_vp
     compute_ts_along_coord
-    compute_directional_stats
     project_to_vertical
     find_rng_index
     get_target_elevations
@@ -38,9 +37,12 @@ from ..core.transforms import antenna_to_cartesian
 from ..io.common import make_time_unit_str
 from ..util.xsect import cross_section_rhi, cross_section_ppi
 from ..util.datetime_utils import datetime_from_radar
+from ..util.circular_stats import compute_directional_stats
+from ..util.radar_utils import ma_broadcast_to
 
 
-def quasi_vertical_profile(radar, desired_angle=None, fields=None, gatefilter=None):
+def quasi_vertical_profile(radar, desired_angle=None, fields=None,
+                           gatefilter=None):
 
     """
     Quasi Vertical Profile.
@@ -73,23 +75,24 @@ def quasi_vertical_profile(radar, desired_angle=None, fields=None, gatefilter=No
     References
     ----------
     Troemel, S., M. Kumjian, A. Ryzhkov, and C. Simmer, 2013: Backscatter
-    differential phase - estimation and variability. J Appl. Meteor. Clim..
+    differential phase - estimation and variability. J Appl. Meteor. Clim.
     52, 2529 - 2548.
 
-    Troemel, S., A. Ryzhkov, P. Zhang, and C. Simmer, 2014: Investigations
-    of backscatter differential phase in the melting layer. J. Appl. Meteorol.
-    Clim. 54, 2344 - 2359.
+    Troemel, S., A. Ryzhkov, P. Zhang, and C. Simmer, 2014:
+    Investigations of backscatter differential phase in the melting layer. J.
+    Appl. Meteorol. Clim. 54, 2344 - 2359.
 
     Ryzhkov, A., P. Zhang, H. Reeves, M. Kumjian, T. Tschallener, S. Tromel,
-    C. Simmer, 2015: Quasi-vertical profiles - a new way to look at polarimetric
-    radar data. Submitted to J. Atmos. Oceanic Technol.
+    C. Simmer, 2015: Quasi-vertical profiles - a new way to look at
+    polarimetric radar data. Submitted to J. Atmos. Oceanic Technol.
 
     """
 
     # Creating an empty dictonary
     qvp = {}
 
-    # Setting the desired radar angle and getting index value for desired radar angle
+    # Setting the desired radar angle and getting index value for desired
+    # radar angle
     if desired_angle is None:
         desired_angle = 20.0
     index = abs(radar.fixed_angle['data'] - desired_angle).argmin()
@@ -110,26 +113,26 @@ def quasi_vertical_profile(radar, desired_angle=None, fields=None, gatefilter=No
             # If none is defined goes to else statement
             if gatefilter is not None:
                 get_fields = radar.get_field(index, field)
-                mask_fields = np.ma.masked_where(gatefilter.gate_excluded[radar_slice],
-                                                 get_fields)
+                mask_fields = np.ma.masked_where(
+                    gatefilter.gate_excluded[radar_slice], get_fields)
                 radar_fields = np.ma.mean(mask_fields, axis=0)
             else:
                 radar_fields = radar.get_field(index, field).mean(axis=0)
 
-            qvp.update({field:radar_fields})
+            qvp.update({field: radar_fields})
 
     else:
         # Filtering data based on defined gatefilter
         # If none is defined goes to else statement
         if gatefilter is not None:
             get_field = radar.get_field(index, fields)
-            mask_field = np.ma.masked_where(gatefilter.gate_excluded[radar_slice],
-                                            get_field)
+            mask_field = np.ma.masked_where(
+                gatefilter.gate_excluded[radar_slice], get_field)
             radar_field = np.ma.mean(mask_field, axis=0)
         else:
             radar_field = radar.get_field(index, fields).mean(axis=0)
 
-        qvp.update({fields:radar_field})
+        qvp.update({fields: radar_field})
 
     # Adding range, time, and height fields
     qvp.update({'range': radar.range['data'], 'time': radar.time})
@@ -451,8 +454,8 @@ def compute_evp(radar, field_names, lon, lat, ref_time=None,
     Reference
     ---------
     Kaltenboeck R., Ryzhkov A. 2016: A freezing rain storm explored with a
-    C-band polarimetric weather radar using the QVP methodology. Meteorologische
-    Zeitschrift vol. 26 pp 207-222
+    C-band polarimetric weather radar using the QVP methodology.
+    Meteorologische Zeitschrift vol. 26 pp 207-222
 
     """
     if avg_type not in ('mean', 'median'):
@@ -789,7 +792,7 @@ def compute_vp(radar, field_names, lon, lat, ref_time=None,
     return qvp
 
 
-def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
+def compute_ts_along_coord(radar, field_names, mode='ALONG_AZI',
                            fixed_range=None, fixed_azimuth=None,
                            fixed_elevation=None, ang_tol=1., rng_tol=50.,
                            value_start=None, value_stop=None, ref_time=None,
@@ -802,8 +805,8 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
     ----------
     radar : Radar
         Radar object used.
-    field_name : str
-        Name of the field
+    field_names : str
+        list of field names
     mode : str
         coordinate to extract data along. Can be ALONG_AZI, ALONG_ELE or
         ALONG_RNG
@@ -865,13 +868,16 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
         # fixed_angle: elevation
         # elevation: elevation
         # azimuth: azimuth
-        rng_values, vals, _, _ = get_data_along_rng(
-            radar, field_name, [fixed_elevation], [fixed_azimuth],
-            ang_tol=ang_tol, rmin=value_start, rmax=value_stop)
+        vals_dict = {}
+        for field_name in field_names:
+            rng_values, vals, _, _ = get_data_along_rng(
+                radar, field_name, [fixed_elevation], [fixed_azimuth],
+                ang_tol=ang_tol, rmin=value_start, rmax=value_stop)
 
-        if vals.size == 0:
-            warn('No data found')
-            return None
+            if vals.size == 0:
+                warn('No data found')
+                return None
+            vals_dict.update({field_name: vals})
         fixed_angle = fixed_elevation
         elevation = fixed_elevation
         azimuth = fixed_azimuth
@@ -881,14 +887,17 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
         # fixed_angle : elevation
         # elevation : elevation
         # azimuth : range
-        rng_values, vals, _, _ = get_data_along_azi(
-            radar, field_name, [fixed_range], [fixed_elevation],
-            rng_tol=rng_tol, ang_tol=ang_tol, azi_start=value_start,
-            azi_stop=value_stop)
+        vals_dict = {}
+        for field_name in field_names:
+            rng_values, vals, _, _ = get_data_along_azi(
+                radar, field_name, [fixed_range], [fixed_elevation],
+                rng_tol=rng_tol, ang_tol=ang_tol, azi_start=value_start,
+                azi_stop=value_stop)
 
-        if vals.size == 0:
-            warn('No data found')
-            return None
+            if vals.size == 0:
+                warn('No data found')
+                return None
+            vals_dict.update({field_name: vals})
         fixed_angle = fixed_elevation
         elevation = fixed_elevation
         azimuth = fixed_range
@@ -898,14 +907,17 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
         # fixed_angle : azimuth
         # elevation : range
         # azimuth : azimuth
-        rng_values, vals, _, _ = get_data_along_ele(
-            radar, field_name, [fixed_range], [fixed_azimuth],
-            rng_tol=rng_tol, ang_tol=ang_tol, ele_min=value_start,
-            ele_max=value_stop)
+        vals_dict = {}
+        for field_name in field_names:
+            rng_values, vals, _, _ = get_data_along_ele(
+                radar, field_name, [fixed_range], [fixed_azimuth],
+                rng_tol=rng_tol, ang_tol=ang_tol, ele_min=value_start,
+                ele_max=value_stop)
 
-        if vals.size == 0:
-            warn('No data found')
-            return None
+            if vals.size == 0:
+                warn('No data found')
+                return None
+            vals_dict.update({field_name: vals})
         fixed_angle = fixed_azimuth
         elevation = fixed_range
         azimuth = fixed_azimuth
@@ -913,7 +925,7 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
 
     if acoord is None:
         acoord = _create_along_coord_object(
-            radar, [field_name], rng_values, fixed_angle, mode,
+            radar, field_names, rng_values, fixed_angle, mode,
             start_time=ref_time)
 
     if not np.allclose(rng_values, acoord.range['data'], rtol=1e5, atol=atol):
@@ -927,52 +939,18 @@ def compute_ts_along_coord(radar, field_name, mode='ALONG_AZI',
         acoord, ref_time, elevation, azimuth)
 
     # Put data in radar object
-    if np.size(acoord.fields[field_name]['data']) == 0:
-        acoord.fields[field_name]['data'] = vals.reshape(1, acoord.ngates)
-    else:
-        acoord.fields[field_name]['data'] = np.ma.concatenate(
-            (acoord.fields[field_name]['data'],
-             vals.reshape(1, acoord.ngates)))
+    for field_name in field_names:
+        if np.size(acoord.fields[field_name]['data']) == 0:
+            acoord.fields[field_name]['data'] = vals_dict[field_name].reshape(
+                1, acoord.ngates)
+        else:
+            acoord.fields[field_name]['data'] = np.ma.concatenate(
+                (acoord.fields[field_name]['data'],
+                 vals_dict[field_name].reshape(1, acoord.ngates)))
 
     return acoord
 
 
-def compute_directional_stats(field, avg_type='mean', nvalid_min=1, axis=0):
-    """
-    Computes the mean or the median along one of the axis (ray or range)
-
-    Parameters
-    ----------
-    field : ndarray
-        the radar field
-    avg_type :str
-        the type of average: 'mean' or 'median'
-    nvalid_min : int
-        the minimum number of points to consider the stats valid. Default 1
-    axis : int
-        the axis along which to compute (0=ray, 1=range)
-
-    Returns
-    -------
-    values : ndarray 1D
-        The resultant statistics
-    nvalid : ndarray 1D
-        The number of valid points used in the computation
-
-    """
-    if avg_type == 'mean':
-        values = np.ma.mean(field, axis=axis)
-    else:
-        values = np.ma.median(field, axis=axis)
-
-    # Set to non-valid if there is not a minimum number of valid gates
-    valid = np.logical_not(np.ma.getmaskarray(field))
-    nvalid = np.sum(valid, axis=0, dtype=int)
-    values[nvalid < nvalid_min] = np.ma.masked
-
-    return values, nvalid
-
-
 def project_to_vertical(data_in, data_height, grid_height, interp_kind='none',
                         fill_value=-9999.):
     """
@@ -1217,24 +1195,29 @@ def get_data_along_rng(radar, field_name, fix_elevations, fix_azimuths,
     yvals = []
     valid_azi = []
     valid_ele = []
-    if radar.scan_type == 'ppi':
+    if radar.scan_type in ('ppi', 'vertical_pointing'):
         for ele, azi in zip(fix_elevations, fix_azimuths):
-            ind_sweep = find_ang_index(
-                radar.fixed_angle['data'], ele, ang_tol=ang_tol)
-            if ind_sweep is None:
-                warn('No elevation angle found for fix_elevation '+str(ele))
-                continue
-            new_dataset = radar.extract_sweeps([ind_sweep])
+            if radar.scan_type == 'vertical_pointing':
+                dataset_line = deepcopy(radar)
+                xvals.extend(x)
+            else:
+                ind_sweep = find_ang_index(
+                    radar.fixed_angle['data'], ele, ang_tol=ang_tol)
+                if ind_sweep is None:
+                    warn('No elevation angle found for fix_elevation '+str(ele))
+                    continue
 
-            try:
-                dataset_line = cross_section_ppi(
-                    new_dataset, [azi], az_tol=ang_tol)
-            except EnvironmentError:
-                warn(' No data found at azimuth '+str(azi) +
-                     ' and elevation '+str(ele))
-                continue
+                new_dataset = radar.extract_sweeps([ind_sweep])
+
+                try:
+                    dataset_line = cross_section_ppi(
+                        new_dataset, [azi], az_tol=ang_tol)
+                except EnvironmentError:
+                    warn(' No data found at azimuth '+str(azi) +
+                        ' and elevation '+str(ele))
+                    continue
+                xvals.append(x)
             yvals.append(dataset_line.fields[field_name]['data'][0, rng_mask])
-            xvals.append(x)
             valid_azi.append(dataset_line.azimuth['data'][0])
             valid_ele.append(dataset_line.elevation['data'][0])
     else:
@@ -1568,7 +1551,8 @@ def _create_along_coord_object(radar, field_names, rng_values, fixed_angle,
         acoord.rays_are_indexed['data'] = np.array(
             [acoord.rays_are_indexed['data'][0]])
     if acoord.ray_angle_res is not None:
-        acoord.ray_angle_res['data'] = np.array([acoord.ray_angle_res['data'][0]])
+        acoord.ray_angle_res['data'] = np.array(
+            [acoord.ray_angle_res['data'][0]])
 
     acoord.fixed_angle['data'] = np.array([fixed_angle], dtype='float64')
 
@@ -1614,7 +1598,7 @@ def _update_qvp_metadata(qvp, ref_time, lon, lat, elev=90.):
         np.ones((qvp.nrays, qvp.ngates), dtype='float64')*lon)
     qvp.gate_latitude['data'] = (
         np.ones((qvp.nrays, qvp.ngates), dtype='float64')*lat)
-    qvp.gate_altitude['data'] = np.broadcast_to(
+    qvp.gate_altitude['data'] = ma_broadcast_to(
         qvp.range['data'], (qvp.nrays, qvp.ngates))
 
     return qvp
