@@ -7,14 +7,17 @@ Functions for working radar instances.
 .. autosummary::
     :toctree: generated/
 
+    compute_antenna_diagram
     is_vpt
     to_vpt
+    compute_azimuthal_average
     join_radar
     join_spectra
     cut_radar
     cut_radar_spectra
     radar_from_spectra
     interpol_spectra
+    find_neighbour_gates
     ma_broadcast_to
 
 """
@@ -26,11 +29,43 @@ import numpy as np
 from scipy.interpolate import interp1d
 from netCDF4 import date2num
 
-from ..config import get_fillvalue
 from ..core import Radar
+from ..config import get_metadata, get_fillvalue
+from . import compute_directional_stats
+from . import cross_section_rhi, get_target_elevations
 from . import datetime_utils
 
 
+def compute_antenna_diagram(npts_diagram=81, beam_factor=2., beam_width=1.):
+    """
+    Computes the antenna diagram. It is assumed a parabolic antenna diagram.
+    The diagram is computed between -beam_factor*beam_width/2 and
+    beam_factor*beam_width/2
+
+    Parameters
+    ----------
+    npts_diagram : int
+        The number of points that that the antenna diagram will have
+    beam_factor : float
+        the factor by which the antenna beam width is multiplied
+
+    Returns
+    -------
+    ang_diag : array of floats
+        The points where the antenna diagram is defined, with origin the
+        center of the diagram (deg)
+    weights_diag : array of floats
+        the weights at each point
+
+    """
+    slope = 2.*beam_factor/(npts_diagram-1.)
+    x = slope*np.arange(npts_diagram)-beam_factor
+    weights_diag = np.power(2., -2*x*x)
+    step = (beam_factor*beam_width)/(npts_diagram-1)
+    ang_diag = np.arange(npts_diagram)*step-beam_factor*beam_width/2.
+    return ang_diag, weights_diag
+
+
 def is_vpt(radar, offset=0.5):
     """
     Determine if a Radar appears to be a vertical pointing scan.
@@ -122,6 +157,163 @@ def to_vpt(radar, single_scan=True):
     # radar.scan_rate
 
 
+def compute_azimuthal_average(radar, field_names, angle=None, delta_azi=None,
+                              avg_type='mean', nvalid_min=1, lin_trans=None):
+    """
+    Computes the azimuthal average
+
+    Parameters
+    ----------
+    radar : Radar object
+        input radar objects
+    field_names : list of str
+        name of the fields to include in the average
+    angle : float or None
+        The center angle to average. If not set or set to -1 all
+        available azimuth angles will be used
+    delta_azi : float
+        The angle span to average. If not set or set to -1 all the
+        available azimuth angles will be used
+    avg_type : str
+        Average type. Can be mean or median. Default mean
+    nvalid_min : int
+        the minimum number of valid points to consdier the average valid.
+        Default 1
+    lin_trans : dict or None
+        If a dictionary, specifies which fields have to be averaged in linear
+        units
+
+    Returns
+    -------
+    radar_rhi : Radar object
+        new radar object where azimuths have been averaged
+    """
+    # keep only fields present in radar object
+    field_names_aux = []
+    nfields_available = 0
+    if avg_type == 'mean':
+        lin_trans_aux = dict()
+    for field_name in field_names:
+        if field_name not in radar.fields:
+            warn('Field name '+field_name+' not available in radar object')
+            continue
+        field_names_aux.append(field_name)
+        nfields_available += 1
+        if avg_type != 'mean':
+            continue
+        lin_trans_aux.update({field_name: False})
+        if lin_trans is None:
+            continue
+        if field_name in lin_trans:
+            lin_trans_aux[field_name] = lin_trans[field_name]
+
+    if nfields_available == 0:
+        warn("Fields not available in radar data")
+        return None
+
+    # check input parameters
+    if avg_type not in ('mean', 'median'):
+        warn('Unsuported statistics '+avg_type)
+        return None
+
+    if delta_azi == -1:
+        delta_azi = None
+    if angle == -1:
+        angle = None
+
+    radar_aux = copy.deepcopy(radar)
+    # transform radar into ppi over the required elevation
+    if radar_aux.scan_type == 'rhi':
+        target_elevations, el_tol = get_target_elevations(radar_aux)
+        radar_ppi = cross_section_rhi(
+            radar_aux, target_elevations, el_tol=el_tol)
+    elif radar_aux.scan_type == 'ppi':
+        radar_ppi = radar_aux
+    else:
+        warn('Error: unsupported scan type.')
+        return None
+
+    # range, metadata, radar position are the same as the original
+    # time
+    radar_rhi = copy.deepcopy(radar)
+    radar_rhi.fields = dict()
+    radar_rhi.scan_type = 'rhi'
+    radar_rhi.sweep_number['data'] = np.array([0])
+    radar_rhi.sweep_mode['data'] = np.array(['rhi'])
+    radar_rhi.fixed_angle['data'] = np.array([0])
+    radar_rhi.sweep_start_ray_index['data'] = np.array([0])
+    radar_rhi.sweep_end_ray_index['data'] = np.array([radar_ppi.nsweeps-1])
+    radar_rhi.rays_per_sweep['data'] = np.array([radar_ppi.nsweeps])
+    radar_rhi.azimuth['data'] = np.ones(radar_ppi.nsweeps)
+    radar_rhi.elevation['data'] = radar_ppi.fixed_angle['data']
+    radar_rhi.time['data'] = np.zeros(radar_ppi.nsweeps)
+    radar_rhi.nrays = radar_ppi.fixed_angle['data'].size
+    radar_rhi.nsweeps = 1
+    radar_rhi.rays_are_indexed = None
+    radar_rhi.ray_angle_res = None
+
+    # average radar data
+    if angle is None:
+        fixed_angle = np.zeros(radar_ppi.nsweeps)
+
+    fields_dict = dict()
+    for field_name in field_names_aux:
+        fields_dict.update(
+            {field_name: get_metadata(field_name)})
+        fields_dict[field_name]['data'] = np.ma.masked_all(
+            (radar_ppi.nsweeps, radar_ppi.ngates))
+    fields_dict.update(
+            {'number_of_samples': get_metadata('number_of_samples')})
+    fields_dict['number_of_samples']['data'] = np.ma.masked_all(
+            (radar_ppi.nsweeps, radar_ppi.ngates))
+
+    for sweep in range(radar_ppi.nsweeps):
+        radar_aux = copy.deepcopy(radar_ppi)
+        radar_aux = radar_aux.extract_sweeps([sweep])
+
+        # find neighbouring gates to be selected
+        inds_ray, inds_rng = find_neighbour_gates(
+            radar_aux, angle, None, delta_azi=delta_azi, delta_rng=None)
+
+        if angle is None:
+            fixed_angle[sweep] = np.median(
+                radar_aux.azimuth['data'][inds_ray])
+
+        # get time as the mean of the time of the selected rays
+        radar_rhi.time['data'][sweep] = np.mean(
+            radar_aux.time['data'][inds_ray])
+
+        # keep only data we are interested in
+        for field_name in field_names_aux:
+            field_aux = radar_aux.fields[field_name]['data'][:, inds_rng]
+            field_aux = field_aux[inds_ray, :]
+            if avg_type == 'mean':
+                if lin_trans_aux[field_name]:
+                    field_aux = np.ma.power(10., 0.1*field_aux)
+
+            vals, nvalid = compute_directional_stats(
+                field_aux, avg_type=avg_type, nvalid_min=nvalid_min, axis=0)
+
+            if avg_type == 'mean':
+                if lin_trans_aux[field_name]:
+                    vals = 10.*np.ma.log10(vals)
+
+            fields_dict[field_name]['data'][sweep, :] = vals
+            fields_dict['number_of_samples']['data'][sweep, :] = nvalid
+
+    if angle is None:
+        radar_rhi.fixed_angle['data'] = np.array([np.mean(fixed_angle)])
+    else:
+        radar_rhi.fixed_angle['data'] = np.array([angle])
+    radar_rhi.azimuth['data'] *= radar_rhi.fixed_angle['data'][0]
+
+    for field_name in field_names_aux:
+        radar_rhi.add_field(field_name, fields_dict[field_name])
+    radar_rhi.add_field('number_of_samples', fields_dict['number_of_samples'])
+
+    return radar_rhi
+
+
 def join_radar(radar1, radar2):
     """
     Combine two radar instances into one.
@@ -365,8 +557,8 @@ def join_spectra(spectra1, spectra2):
     r1num = datetime_utils.datetimes_from_radar(spectra1, epoch=True)
     r2num = datetime_utils.datetimes_from_radar(spectra2, epoch=True)
     new_spectra.time['data'] = date2num(
-        np.append(r1num, r2num), datetime_utils.EPOCH_UNITS)
-    new_spectra.time['units'] = datetime_utils.EPOCH_UNITS
+        np.append(r1num, r2num), datetime_units.EPOCH_UNITS)
+    new_spectra.time['units'] = datetime_units.EPOCH_UNITS
     new_spectra.nrays = len(new_spectra.time['data'])
 
     fields_to_remove = []
@@ -411,9 +603,12 @@ def join_spectra(spectra1, spectra2):
         if (lat1 != lat2) or (lon1 != lon2) or (alt1 != alt2):
             ones1 = np.ones(len(spectra1.time['data']), dtype='float32')
             ones2 = np.ones(len(spectra2.time['data']), dtype='float32')
-            new_spectra.latitude['data'] = np.append(ones1 * lat1, ones2 * lat2)
-            new_spectra.longitude['data'] = np.append(ones1 * lon1, ones2 * lon2)
-            new_spectra.latitude['data'] = np.append(ones1 * alt1, ones2 * alt2)
+            new_spectra.latitude['data'] = np.append(
+                ones1 * lat1, ones2 * lat2)
+            new_spectra.longitude['data'] = np.append(
+                ones1 * lon1, ones2 * lon2)
+            new_spectra.latitude['data'] = np.append(
+                ones1 * alt1, ones2 * alt2)
         else:
             new_spectra.latitude['data'] = spectra1.latitude['data']
             new_spectra.longitude['data'] = spectra1.longitude['data']
@@ -466,7 +661,8 @@ def cut_radar(radar, field_names, rng_min=None, rng_max=None, ele_min=None,
         rng_max = np.max(radar_aux.range['data'])
 
     ind_rng = np.where(np.logical_and(
-        radar_aux.range['data'] >= rng_min, radar_aux.range['data'] <= rng_max))[0]
+        radar_aux.range['data'] >= rng_min,
+        radar_aux.range['data'] <= rng_max))[0]
 
     if ind_rng.size == 0:
         warn('No range bins between '+str(rng_min)+' and '+str(rng_max)+' m')
@@ -550,11 +746,29 @@ def cut_radar(radar, field_names, rng_min=None, rng_max=None, ele_min=None,
     sweep_end_inds = copy.deepcopy(radar_aux.sweep_end_ray_index['data'])
 
     nrays = 0
+    ind_rays_aux = []
     for j in range(radar_aux.nsweeps):
-        # get azimuth indices for this elevation
-        rays_in_sweep = np.size(
-            ind_rays[np.logical_and(ind_rays >= sweep_start_inds[j],
-                                    ind_rays <= sweep_end_inds[j])])
+        # get ray indices for this sweep
+        ind_rays_sweep = ind_rays[np.logical_and(
+            ind_rays >= sweep_start_inds[j], ind_rays <= sweep_end_inds[j])]
+        # order rays
+        if radar_aux.scan_type == 'ppi':
+            ind = np.argsort(radar_aux.azimuth['data'][ind_rays_sweep])
+            ind_rays_sweep = ind_rays_sweep[ind]
+            # avoid large gaps in data
+            azimuths = radar_aux.azimuth['data'][ind_rays_sweep]
+            azi_steps = azimuths[1:]-azimuths[:-1]
+            ind_gap = np.where(azi_steps > 2*np.median(azi_steps))[0]
+            if ind_gap.size > 0:
+                ind_rays_sweep = np.append(
+                    ind_rays_sweep[ind_gap[0]+1:],
+                    ind_rays_sweep[:ind_gap[0]+1])
+            ind_rays_aux.extend(ind_rays_sweep)
+        else:
+            ind = np.argsort(radar_aux.elevation['data'][ind_rays_sweep])
+            ind_rays_aux.extend(ind_rays_sweep[ind])
+
+        rays_in_sweep = ind_rays_sweep.size
         radar_aux.rays_per_sweep['data'][j] = rays_in_sweep
         if j == 0:
             radar_aux.sweep_start_ray_index['data'][j] = 0
@@ -565,6 +779,8 @@ def cut_radar(radar, field_names, rng_min=None, rng_max=None, ele_min=None,
             radar_aux.sweep_start_ray_index['data'][j]+rays_in_sweep-1)
         nrays += rays_in_sweep
 
+    ind_rays = np.array(ind_rays_aux)
+
     # Update metadata
     radar_aux.range['data'] = radar_aux.range['data'][ind_rng]
     radar_aux.time['data'] = radar_aux.time['data'][ind_rays]
@@ -646,7 +862,8 @@ def cut_radar_spectra(radar, field_names, rng_min=None, rng_max=None,
         rng_max = np.max(radar_aux.range['data'])
 
     ind_rng = np.where(np.logical_and(
-        radar_aux.range['data'] >= rng_min, radar_aux.range['data'] <= rng_max))[0]
+        radar_aux.range['data'] >= rng_min,
+        radar_aux.range['data'] <= rng_max))[0]
 
     if ind_rng.size == 0:
         warn('No range bins between '+str(rng_min)+' and '+str(rng_max)+' m')
@@ -905,9 +1122,57 @@ def interpol_spectra(psr, kind='linear', fill_value=0.):
     return psr_interp
 
 
+def find_neighbour_gates(radar, azi, rng, delta_azi=None, delta_rng=None):
+    """
+    Find the neighbouring gates within +-delta_azi and +-delta_rng
+
+    Parameters
+    ----------
+    radar : radar object
+        the radar object
+    azi, rng : float
+        The azimuth [deg] and range [m] of the central gate
+    delta_azi, delta_rng : float
+        The extend where to look for
+
+    Returns
+    -------
+    inds_ray_aux, ind_rng_aux : int
+        The indices (ray, rng) of the neighbouring gates
+
+    """
+    # find gates close to lat lon point
+    if delta_azi is None:
+        inds_ray = np.ma.arange(radar.azimuth['data'].size)
+    else:
+        azi_max = azi+delta_azi
+        azi_min = azi-delta_azi
+        if azi_max > 360.:
+            azi_max -= 360.
+        if azi_min < 0.:
+            azi_min += 360.
+        if azi_max > azi_min:
+            inds_ray = np.where(np.logical_and(
+                radar.azimuth['data'] < azi_max,
+                radar.azimuth['data'] > azi_min))[0]
+        else:
+            inds_ray = np.where(np.logical_or(
+                radar.azimuth['data'] > azi_min,
+                radar.azimuth['data'] < azi_max))[0]
+    if delta_rng is None:
+        inds_rng = np.ma.arange(radar.range['data'].size)
+    else:
+        inds_rng = np.where(np.logical_and(
+            radar.range['data'] < rng+delta_rng,
+            radar.range['data'] > rng-delta_rng))[0]
+
+    return inds_ray, inds_rng
+
+
 def ma_broadcast_to(array, tup):
     """
-    Is used to guarantee that a masked array can be broadcasted without loosing the mask
+    Is used to guarantee that a masked array can be broadcasted without
+    loosing the mask
 
     Parameters
     ----------
@@ -927,6 +1192,8 @@ def ma_broadcast_to(array, tup):
         initial_mask = np.ma.getmask(array)
         initial_fill_value = array.fill_value
         broadcasted_mask = np.broadcast_to(initial_mask, tup)
-        return np.ma.array(broadcasted_array, mask=broadcasted_mask, fill_value=initial_fill_value)
+        return np.ma.array(
+            broadcasted_array, mask=broadcasted_mask,
+            fill_value=initial_fill_value)
 
     return broadcasted_array
